@@ -2,5 +2,5 @@
 class GlideParams(object):
 
 
-    def __init__(self, args):
+    def generate_glide_params(self, args):
         self.precision_glide = args.precision_glide
