@@ -1,7 +1,7 @@
 from ._version import get_versions
+
 __version__ = get_versions()['version']
 del get_versions
 
-__author__ = "Daniel Soler Viladrich"
-
-__email__ = "daniel.soler@nostrumbiodiscovery.com"
+__author__ = "Nostrum Biodiscovery"
+__email__ = "pelesupport@nostrumbiodiscovery.com"
