@@ -1,15 +1,15 @@
 from dataclasses import dataclass
 import pele_platform.Adaptive.simulation as si
 import pele_platform.Utilities.Helpers.helpers as hp
-import pele_platform.Utilities.Parameters.pele_env as pv
+import pele_platform.Utilities.Parameters.parameters as pv
 
 
 @dataclass
 class GpcrLauncher:
 
-    args: pv.EnviroBuilder 
+    args: pv.ParametersBuilder
 
-    def run_gpcr_simulation(self) -> pv.EnviroBuilder:
+    def run_gpcr_simulation(self) -> pv.ParametersBuilder:
         # Set parameters for GPCR and launch simulation
         self._set_parameters()
         simulation_parameters = si.run_adaptive(self.args)
